/* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include "common/scummsys.h"

#ifdef __wii__

#include "backends/events/wiisdl/wiisdl-events.h"
#include "backends/platform/sdl/sdl.h"
#include "engines/engine.h"

#include "common/util.h"
#include "common/events.h"

enum {
	BTN_WIIMOTE_A		= 0,
	BTN_WIIMOTE_B		= 1,
	BTN_WIIMOTE_1		= 2,
	BTN_WIIMOTE_2		= 3,
	BTN_WIIMOTE_MINUS	= 4,
	BTN_WIIMOTE_PLUS	= 5,
	BTN_WIIMOTE_HOME	= 6,
	BTN_WIIMOTE_Z		= 7,
	BTN_WIIMOTE_C		= 8,

	BTN_WIIMOTE_CLASSIC_A		= 9,
	BTN_WIIMOTE_CLASSIC_B		= 10,
	BTN_WIIMOTE_CLASSIC_X		= 11,
	BTN_WIIMOTE_CLASSIC_Y		= 12,
	BTN_WIIMOTE_CLASSIC_L		= 13,
	BTN_WIIMOTE_CLASSIC_R		= 14,
	BTN_WIIMOTE_CLASSIC_ZL		= 15,
	BTN_WIIMOTE_CLASSIC_ZR		= 16,
	BTN_WIIMOTE_CLASSIC_MINUS	= 17,
	BTN_WIIMOTE_CLASSIC_PLUS	= 18,
	BTN_WIIMOTE_CLASSIC_HOME	= 19
};

static void pushKeyboardEvent( SDLKey key, bool pressed )
{
  SDL_Event event;
  event.type = pressed ? SDL_KEYDOWN : SDL_KEYUP;
  event.key.state = pressed ? SDL_PRESSED : SDL_RELEASED;
  event.key.keysym.scancode = 0;
  event.key.keysym.sym = key;
  event.key.keysym.unicode = 0;
  SDL_PushEvent( &event );
}

bool WIISdlEventSource::handleJoyHat(SDL_Event &ev, Common::Event &event) {
	
	// Far from perfect right now. The directions can be mixed and the character will run in circle.
	// But, well... it "works" at least.
	switch (ev.jhat.value) {
	case SDL_HAT_CENTERED:
		event.type = Common::EVENT_KEYUP;

		event.kbd.keycode = Common::KEYCODE_UP;
		event.kbd.ascii = mapKey(SDLK_UP, (SDLMod)ev.key.keysym.mod, 0);
		g_system->getEventManager()->pushEvent(event);

		event.kbd.keycode = Common::KEYCODE_DOWN;
		event.kbd.ascii = mapKey(SDLK_DOWN, (SDLMod)ev.key.keysym.mod, 0);
		g_system->getEventManager()->pushEvent(event);

		event.kbd.keycode = Common::KEYCODE_LEFT;
		event.kbd.ascii = mapKey(SDLK_LEFT, (SDLMod)ev.key.keysym.mod, 0);
		g_system->getEventManager()->pushEvent(event);

		event.kbd.keycode = Common::KEYCODE_RIGHT;
		event.kbd.ascii = mapKey(SDLK_RIGHT, (SDLMod)ev.key.keysym.mod, 0);
		g_system->getEventManager()->pushEvent(event);
		break;
	case SDL_HAT_UP:
		event.type = Common::EVENT_KEYDOWN;
		event.kbd.keycode = Common::KEYCODE_UP;
		event.kbd.ascii = mapKey(SDLK_UP, (SDLMod)ev.key.keysym.mod, 0);
		ev.jhat.value=SDL_HAT_CENTERED;
	break;
	case SDL_HAT_RIGHT:
		event.type = Common::EVENT_KEYDOWN;
		event.kbd.keycode = Common::KEYCODE_RIGHT;
		event.kbd.ascii = mapKey(SDLK_RIGHT, (SDLMod)ev.key.keysym.mod, 0);
		ev.jhat.value=SDL_HAT_CENTERED;
	break;
	case SDL_HAT_DOWN:
		event.type = Common::EVENT_KEYDOWN;
		event.kbd.keycode = Common::KEYCODE_DOWN;
		event.kbd.ascii = mapKey(SDLK_DOWN, (SDLMod)ev.key.keysym.mod, 0);
		ev.jhat.value=SDL_HAT_CENTERED;
	break;
	case SDL_HAT_LEFT:
		event.type = Common::EVENT_KEYDOWN;
		event.kbd.keycode = Common::KEYCODE_LEFT;
		event.kbd.ascii = mapKey(SDLK_LEFT, (SDLMod)ev.key.keysym.mod, 0);
		ev.jhat.value=SDL_HAT_CENTERED;
	break;
	case SDL_HAT_RIGHTUP:
		event.type = Common::EVENT_KEYUP;
		event.kbd.keycode = Common::KEYCODE_RIGHT;
		event.kbd.ascii = mapKey(SDLK_RIGHT, (SDLMod)ev.key.keysym.mod, 0);
		ev.jhat.value=SDL_HAT_CENTERED;
	break;
	case SDL_HAT_RIGHTDOWN:	
	break;
	case SDL_HAT_LEFTUP:
		event.type = Common::EVENT_KEYUP;
		event.kbd.keycode = Common::KEYCODE_LEFT;
		event.kbd.ascii = mapKey(SDLK_LEFT, (SDLMod)ev.key.keysym.mod, 0);
		ev.jhat.value=SDL_HAT_CENTERED;
	break;
	case SDL_HAT_LEFTDOWN:
	break;
	}
	return true;
}


bool WIISdlEventSource::handleJoyAxisMotion(SDL_Event &ev, Common::Event &event) {
	int axis = ev.jaxis.value;
#define JOY_DEADZONE 3200
#define JOY_XAXIS 0
#define JOY_YAXIS 1

	if ( axis > JOY_DEADZONE) {
		axis -= JOY_DEADZONE;
	} else if ( axis < -JOY_DEADZONE ) {
		axis += JOY_DEADZONE;
	} else {
		axis = 0;
	}

	if ( ev.jaxis.axis == JOY_XAXIS) {
		if (axis != 0) {
			event.type = Common::EVENT_KEYDOWN;
			if (axis > 0) {
				event.kbd.keycode = Common::KEYCODE_RIGHT;
				event.kbd.ascii = mapKey(SDLK_RIGHT, (SDLMod)ev.key.keysym.mod, 0);
			} else {
				event.kbd.keycode = Common::KEYCODE_LEFT;
				event.kbd.ascii = mapKey(SDLK_LEFT, (SDLMod)ev.key.keysym.mod, 0);
			}
		} else {
			event.type = Common::EVENT_KEYUP;

			event.kbd.keycode = Common::KEYCODE_RIGHT;
			event.kbd.ascii = mapKey(SDLK_RIGHT, (SDLMod)ev.key.keysym.mod, 0);
			g_system->getEventManager()->pushEvent(event);

			event.kbd.keycode = Common::KEYCODE_LEFT;
			event.kbd.ascii = mapKey(SDLK_LEFT, (SDLMod)ev.key.keysym.mod, 0);
		}
	} else if (ev.jaxis.axis == JOY_YAXIS) {
#ifndef JOY_INVERT_Y
		axis = -axis;
#endif
		if (axis != 0) {
			event.type = Common::EVENT_KEYDOWN;
			if (axis > 0) {
				event.kbd.keycode = Common::KEYCODE_UP;
				event.kbd.ascii = mapKey(SDLK_UP, (SDLMod)ev.key.keysym.mod, 0);
			} else {
				event.kbd.keycode = Common::KEYCODE_DOWN;
				event.kbd.ascii = mapKey(SDLK_DOWN, (SDLMod)ev.key.keysym.mod, 0);
			}
		} else {
			event.type = Common::EVENT_KEYUP;
			event.kbd.keycode = Common::KEYCODE_UP;
			event.kbd.ascii = mapKey(SDLK_UP, (SDLMod)ev.key.keysym.mod, 0);
			g_system->getEventManager()->pushEvent(event);

			event.kbd.keycode = Common::KEYCODE_DOWN;
			event.kbd.ascii = mapKey(SDLK_DOWN, (SDLMod)ev.key.keysym.mod, 0);
		}
	}


return true;
}

bool WIISdlEventSource::handleJoyButtonDown(SDL_Event &ev, Common::Event &event) {

	event.kbd.flags = 0;

	// Wiimote buttons
	switch (ev.jbutton.button) {
	case BTN_WIIMOTE_A:
	break;
	case BTN_WIIMOTE_B: 
		pushKeyboardEvent( SDLK_p, true );			// Pick up/Put away
	break;
	case BTN_WIIMOTE_1:
		pushKeyboardEvent( SDLK_KP_ENTER, true );	// Use item/Talk
	break;
	case BTN_WIIMOTE_2:
		pushKeyboardEvent( SDLK_e, true );			// Examine item
	break;
	case BTN_WIIMOTE_MINUS:
		pushKeyboardEvent( SDLK_ESCAPE, true );		// Skip cutscene/Exit inventory
	break;
	case BTN_WIIMOTE_PLUS:
		pushKeyboardEvent( SDLK_i, true );			// Open inventory
	break;
	case BTN_WIIMOTE_HOME:
		event.type = Common::EVENT_MAINMENU;		// Game menu
	break;
	case BTN_WIIMOTE_Z:
		pushKeyboardEvent( SDLK_LSHIFT, true );		// Run
	break;
	case BTN_WIIMOTE_C:
		event.type = Common::EVENT_KEYDOWN;			// Virtual keyboard
		event.kbd.keycode = Common::KEYCODE_F7;
		event.kbd.ascii = mapKey(SDLK_F7, (SDLMod) ev.key.keysym.mod, 0);
	break;

	// Classic controller buttons
	case BTN_WIIMOTE_CLASSIC_A:
		pushKeyboardEvent( SDLK_KP_ENTER, true );	// Use item/Talk/OK for menu and dialogs
	break;
	case BTN_WIIMOTE_CLASSIC_B:
		pushKeyboardEvent( SDLK_p, true );			// Pick up/Put away
	break;
	case BTN_WIIMOTE_CLASSIC_X:
		pushKeyboardEvent( SDLK_e, true );			// Examine item
	break;
	case BTN_WIIMOTE_CLASSIC_Y:
		pushKeyboardEvent( SDLK_i, true );			// Open inventory
	break;
	case BTN_WIIMOTE_CLASSIC_L:
		event.type = Common::EVENT_KEYDOWN;			// Virtual keyboard
		event.kbd.keycode = Common::KEYCODE_F7;
		event.kbd.ascii = mapKey(SDLK_F7, (SDLMod) ev.key.keysym.mod, 0);
	break;
	case BTN_WIIMOTE_CLASSIC_R:
		pushKeyboardEvent( SDLK_LSHIFT, true );		// Run
	break;
	case BTN_WIIMOTE_CLASSIC_ZL:
	break;
	case BTN_WIIMOTE_CLASSIC_ZR:
	break;
	case BTN_WIIMOTE_CLASSIC_MINUS:
		pushKeyboardEvent( SDLK_ESCAPE, true );		// Skip cutscene/Exit inventory
	break;
	case BTN_WIIMOTE_CLASSIC_PLUS:
	break;
	case BTN_WIIMOTE_CLASSIC_HOME:
		event.type = Common::EVENT_MAINMENU;		// Game menu
	break;
	}
	return true;
}

bool WIISdlEventSource::handleJoyButtonUp(SDL_Event &ev, Common::Event &event) {

	event.kbd.flags = 0;

	// Wiimote buttons
	switch (ev.jbutton.button) {
	case BTN_WIIMOTE_A: 
	break;
	case BTN_WIIMOTE_B: 
		pushKeyboardEvent( SDLK_p, false );			// Pick up/Put away
	break;
	case BTN_WIIMOTE_1:
		pushKeyboardEvent( SDLK_KP_ENTER, false );	// Use item/Talk
	break;
	case BTN_WIIMOTE_2:
		pushKeyboardEvent( SDLK_e, false );			// Examine item
	break;
	case BTN_WIIMOTE_MINUS:
		pushKeyboardEvent( SDLK_ESCAPE, false );	// Skip cutscene/Exit inventory
	break;
	case BTN_WIIMOTE_PLUS:
		pushKeyboardEvent( SDLK_i, false );			// Open inventory
	break;
	case BTN_WIIMOTE_HOME:
	break;
	case BTN_WIIMOTE_Z:
		pushKeyboardEvent( SDLK_LSHIFT, false );	// Run
	break;
	case BTN_WIIMOTE_C:
		event.type = Common::EVENT_KEYUP;			// Virtual keyboard
		event.kbd.keycode = Common::KEYCODE_F7;
		event.kbd.ascii = mapKey(SDLK_F7, (SDLMod) ev.key.keysym.mod, 0);
	break;

	// Classic controller buttons
	case BTN_WIIMOTE_CLASSIC_A:
		pushKeyboardEvent( SDLK_KP_ENTER, false );	// Use item/Talk/OK for menu and dialogs
	break;
	case BTN_WIIMOTE_CLASSIC_B:
		pushKeyboardEvent( SDLK_p, false );			// Pick up/Put away
	break;
	case BTN_WIIMOTE_CLASSIC_X:
		pushKeyboardEvent( SDLK_e, false );			// Examine item
	break;
	case BTN_WIIMOTE_CLASSIC_Y:
		pushKeyboardEvent( SDLK_i, false );			// Open inventory
	break;
	case BTN_WIIMOTE_CLASSIC_L:
		event.type = Common::EVENT_KEYUP;			// Virtual keyboard
		event.kbd.keycode = Common::KEYCODE_F7;
		event.kbd.ascii = mapKey(SDLK_F7, (SDLMod) ev.key.keysym.mod, 0);
	break;
	case BTN_WIIMOTE_CLASSIC_R:
		pushKeyboardEvent( SDLK_LSHIFT, false );	// Run
	break;
	case BTN_WIIMOTE_CLASSIC_ZL:
	break;
	case BTN_WIIMOTE_CLASSIC_ZR:
	break;
	case BTN_WIIMOTE_CLASSIC_MINUS:
		pushKeyboardEvent( SDLK_ESCAPE, false );	// Skip cutscene/Exit inventory
	break;
	case BTN_WIIMOTE_CLASSIC_PLUS:
	break;
	case BTN_WIIMOTE_CLASSIC_HOME:
	break;
	}
	return true;
}

#endif
