/* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */
#define FORBIDDEN_SYMBOL_EXCEPTION_chdir
#define FORBIDDEN_SYMBOL_EXCEPTION_getcwd
#define FORBIDDEN_SYMBOL_EXCEPTION_printf

#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include <unistd.h>
#include "common/scummsys.h"
#include <gccore.h>
#include "fat.h"
#include "backends/platform/sdl/wii/wii.h"
#include "backends/plugins/sdl/sdl-provider.h"
#include "base/main.h"

#undef main

extern "C" int SDL_main(int argc, char * argv[]) {
	L2Enhance();
	fatInitDefault();

	char cwd[MAXPATHLEN];

	if (getcwd(cwd, MAXPATHLEN)) {
		size_t len = strlen(cwd);

			if (len > 2 && (cwd[len - 1] == ':' || cwd[len - 2] == ':')) {
				//printf("chdir to default\n");
				chdir("/apps/grim");
			}
	}

	// Create our OSystem instance
	g_system = new OSystem_Wii();
	assert(g_system);

	// Pre initialize the backend
	((OSystem_Wii *)g_system)->init();

#ifdef DYNAMIC_MODULES
	PluginManager::instance().addPluginProvider(new SDLPluginProvider());
#endif

	// Invoke the actual ScummVM main entry point:
	int res = scummvm_main(argc, argv);

	// Free OSystem
	delete (OSystem_Wii *)g_system;

	return res;
}
