/* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 */

#include "common/scummsys.h"
#include "common/config-manager.h"

#ifdef __wii__

#include "backends/platform/sdl/wii/wii.h"
#include "backends/fs/wii/wii-fs-factory.h"
#include "backends/saves/default/default-saves.h"
#include "common/events.h"
#include "backends/events/wiisdl/wiisdl-events.h"

void OSystem_Wii::init() {
	// Initialyze File System Factory
	_fsFactory = new WiiFilesystemFactory();

	// Invoke parent implementation of this method
	OSystem_SDL::init();
}


void OSystem_Wii::initBackend() {
	ConfMan.set("joystick_num", 0);
	ConfMan.set("vkeybdpath", "/apps/grim");
	ConfMan.set("gui_theme", "modern");
	ConfMan.set("themepath", "/apps/grim");
	ConfMan.set("extrapath", "/apps/grim");
	ConfMan.registerDefault("fullscreen", true);

	// Create the savefile manager
	if (_savefileManager == 0)
		_savefileManager = new DefaultSaveFileManager("/apps/grim/saves");

	// Event source
	if (_eventSource == 0)
		_eventSource = new WIISdlEventSource();

	// Invoke parent implementation of this method
	OSystem_SDL::initBackend();
}

Common::WriteStream *OSystem_Wii::createLogFile() {
	Common::FSNode file("/apps/grim/residualvm.log");
	return file.createWriteStream();
}

#endif
